@@ -2,6 +2,7 @@
 from __future__ import annotations
 
 import logging
+from typing import TYPE_CHECKING, Any
 
 from selenium.common.exceptions import (
     InvalidSessionIdException,
@@ -9,6 +10,8 @@ from selenium.common.exceptions import (
     StaleElementReferenceException,
 )
 
+if TYPE_CHECKING:
+    from shadowstep.shadowstep import Shadowstep
 from shadowstep.base import WebDriverSingleton
 from shadowstep.decorators.decorators import fail_safe
 from shadowstep.exceptions.shadowstep_exceptions import ShadowstepException
@@ -17,9 +20,9 @@ from shadowstep.utils.utils import get_current_func_name
 
 class MobileCommands:
 
-    def __init__(self, shadowstep: 'Shadowstep'):
+    def __init__(self, shadowstep: Shadowstep):
         self.shadowstep = shadowstep
-        self.driver = None
+        self.driver: Any = None
         self.logger = logging.getLogger(f"{__name__}.{self.__class__.__name__}")
         
 
@@ -29,7 +32,7 @@ class MobileCommands:
         raise_exception=ShadowstepException,
         exceptions=(NoSuchDriverException, InvalidSessionIdException, StaleElementReferenceException)
     )
-    def activate_app(self, params: dict | list) -> 'Shadowstep':
+    def activate_app(self, params: dict[str, Any] | list[Any]) -> MobileCommands:
         """
         https://github.com/appium/appium-uiautomator2-driver/blob/master/docs/android-mobile-gestures.md#mobile activateapp
         Execute mobile: activateApp command.
@@ -41,7 +44,7 @@ class MobileCommands:
             Shadowstep: Self for method chaining.
         """
         self.logger.debug(f"{get_current_func_name()}")
-        self._execute('mobile: activateApp', params or {})
+        self._execute("mobile: activateApp", params)
         return self
 
 
@@ -51,7 +54,7 @@ class MobileCommands:
         raise_exception=ShadowstepException,
         exceptions=(NoSuchDriverException, InvalidSessionIdException, StaleElementReferenceException)
     )
-    def batteryinfo(self, params: dict | list = None) -> 'Shadowstep':
+    def batteryinfo(self, params: dict[str, Any] | list[Any] | None = None) -> MobileCommands:
         """
         https://github.com/appium/appium-uiautomator2-driver/blob/master/docs/android-mobile-gestures.md#mobile batteryinfo
         Execute mobile: batteryInfo command.
@@ -63,7 +66,7 @@ class MobileCommands:
             Shadowstep: Self for method chaining.
         """
         self.logger.debug(f"{get_current_func_name()}")
-        self._execute('mobile: batteryInfo', params or {})
+        self._execute("mobile: batteryInfo", params)
         return self
 
 
@@ -73,7 +76,7 @@ class MobileCommands:
         raise_exception=ShadowstepException,
         exceptions=(NoSuchDriverException, InvalidSessionIdException, StaleElementReferenceException)
     )
-    def clearelement(self, params: dict | list = None) -> 'Shadowstep':
+    def clearelement(self, params: dict[str, Any] | list[Any] | None = None) -> MobileCommands:
         """
         https://github.com/appium/appium-uiautomator2-driver/blob/master/docs/android-mobile-gestures.md#mobile clearelement
         Execute mobile: clearElement command.
@@ -85,7 +88,7 @@ class MobileCommands:
             Shadowstep: Self for method chaining.
         """
         self.logger.debug(f"{get_current_func_name()}")
-        self._execute('mobile: clearElement', params or {})
+        self._execute("mobile: clearElement", params)
         return self
 
 
@@ -95,7 +98,7 @@ class MobileCommands:
         raise_exception=ShadowstepException,
         exceptions=(NoSuchDriverException, InvalidSessionIdException, StaleElementReferenceException)
     )
-    def deviceinfo(self, params: dict | list = None) -> 'Shadowstep':
+    def deviceinfo(self, params: dict[str, Any] | list[Any] | None = None) -> MobileCommands:
         """
         https://github.com/appium/appium-uiautomator2-driver/blob/master/docs/android-mobile-gestures.md#mobile deviceinfo
         Execute mobile: deviceInfo command.
@@ -107,7 +110,7 @@ class MobileCommands:
             Shadowstep: Self for method chaining.
         """
         self.logger.debug(f"{get_current_func_name()}")
-        self._execute('mobile: deviceInfo', params or {})
+        self._execute("mobile: deviceInfo", params)
         return self
 
 
@@ -117,7 +120,7 @@ class MobileCommands:
         raise_exception=ShadowstepException,
         exceptions=(NoSuchDriverException, InvalidSessionIdException, StaleElementReferenceException)
     )
-    def fingerprint(self, params: dict | list = None) -> 'Shadowstep':
+    def fingerprint(self, params: dict[str, Any] | list[Any] | None = None) -> MobileCommands:
         """
         https://github.com/appium/appium-uiautomator2-driver/blob/master/docs/android-mobile-gestures.md#mobile fingerprint
         Execute mobile: fingerprint command.
@@ -129,7 +132,7 @@ class MobileCommands:
             Shadowstep: Self for method chaining.
         """
         self.logger.debug(f"{get_current_func_name()}")
-        self._execute('mobile: fingerprint', params or {})
+        self._execute("mobile: fingerprint", params)
         return self
 
 
@@ -139,7 +142,7 @@ class MobileCommands:
         raise_exception=ShadowstepException,
         exceptions=(NoSuchDriverException, InvalidSessionIdException, StaleElementReferenceException)
     )
-    def get_clipboard(self, params: dict | list = None) -> 'Shadowstep':
+    def get_clipboard(self, params: dict[str, Any] | list[Any] | None = None) -> MobileCommands:
         """
         https://github.com/appium/appium-uiautomator2-driver/blob/master/docs/android-mobile-gestures.md#mobile getclipboard
         Execute mobile: getClipboard command.
@@ -151,7 +154,7 @@ class MobileCommands:
             Shadowstep: Self for method chaining.
         """
         self.logger.debug(f"{get_current_func_name()}")
-        self._execute('mobile: getClipboard', params or {})
+        self._execute("mobile: getClipboard", params)
         return self
 
 
@@ -161,7 +164,7 @@ class MobileCommands:
         raise_exception=ShadowstepException,
         exceptions=(NoSuchDriverException, InvalidSessionIdException, StaleElementReferenceException)
     )
-    def get_current_activity(self, params: dict | list = None) -> 'Shadowstep':
+    def get_current_activity(self, params: dict[str, Any] | list[Any] | None = None) -> MobileCommands:
         """
         https://github.com/appium/appium-uiautomator2-driver/blob/master/docs/android-mobile-gestures.md#mobile getcurrentactivity
         Execute mobile: getCurrentActivity command.
@@ -173,7 +176,7 @@ class MobileCommands:
             Shadowstep: Self for method chaining.
         """
         self.logger.debug(f"{get_current_func_name()}")
-        self._execute('mobile: getCurrentActivity', params or {})
+        self._execute("mobile: getCurrentActivity", params)
         return self
 
 
@@ -183,7 +186,7 @@ class MobileCommands:
         raise_exception=ShadowstepException,
         exceptions=(NoSuchDriverException, InvalidSessionIdException, StaleElementReferenceException)
     )
-    def get_currentpackage(self, params: dict | list = None) -> 'Shadowstep':
+    def get_currentpackage(self, params: dict[str, Any] | list[Any] | None = None) -> MobileCommands:
         """
         https://github.com/appium/appium-uiautomator2-driver/blob/master/docs/android-mobile-gestures.md#mobile getcurrentpackage
         Execute mobile: getCurrentPackage command.
@@ -195,7 +198,7 @@ class MobileCommands:
             Shadowstep: Self for method chaining.
         """
         self.logger.debug(f"{get_current_func_name()}")
-        self._execute('mobile: getCurrentPackage', params or {})
+        self._execute("mobile: getCurrentPackage", params)
         return self
 
 
@@ -205,7 +208,7 @@ class MobileCommands:
         raise_exception=ShadowstepException,
         exceptions=(NoSuchDriverException, InvalidSessionIdException, StaleElementReferenceException)
     )
-    def get_devicetime(self, params: dict | list = None) -> 'Shadowstep':
+    def get_devicetime(self, params: dict[str, Any] | list[Any] | None = None) -> MobileCommands:
         """
         https://github.com/appium/appium-uiautomator2-driver/blob/master/docs/android-mobile-gestures.md#mobile getdevicetime
         Execute mobile: getDeviceTime command.
@@ -217,7 +220,7 @@ class MobileCommands:
             Shadowstep: Self for method chaining.
         """
         self.logger.debug(f"{get_current_func_name()}")
-        self._execute('mobile: getDeviceTime', params or {})
+        self._execute("mobile: getDeviceTime", params)
         return self
 
 
@@ -227,7 +230,7 @@ class MobileCommands:
         raise_exception=ShadowstepException,
         exceptions=(NoSuchDriverException, InvalidSessionIdException, StaleElementReferenceException)
     )
-    def get_performancedata(self, params: dict | list = None) -> 'Shadowstep':
+    def get_performancedata(self, params: dict[str, Any] | list[Any] | None = None) -> MobileCommands:
         """
         https://github.com/appium/appium-uiautomator2-driver/blob/master/docs/android-mobile-gestures.md#mobile getperformancedata
         Execute mobile: getPerformanceData command.
@@ -239,7 +242,7 @@ class MobileCommands:
             Shadowstep: Self for method chaining.
         """
         self.logger.debug(f"{get_current_func_name()}")
-        self._execute('mobile: getPerformanceData', params or {})
+        self._execute("mobile: getPerformanceData", params)
         return self
 
 
@@ -249,7 +252,7 @@ class MobileCommands:
         raise_exception=ShadowstepException,
         exceptions=(NoSuchDriverException, InvalidSessionIdException, StaleElementReferenceException)
     )
-    def get_performancedatatypes(self, params: dict | list = None) -> 'Shadowstep':
+    def get_performancedatatypes(self, params: dict[str, Any] | list[Any] | None = None) -> MobileCommands:
         """
         https://github.com/appium/appium-uiautomator2-driver/blob/master/docs/android-mobile-gestures.md#mobile getperformancedatatypes
         Execute mobile: getPerformanceDataTypes command.
@@ -261,7 +264,7 @@ class MobileCommands:
             Shadowstep: Self for method chaining.
         """
         self.logger.debug(f"{get_current_func_name()}")
-        self._execute('mobile: getPerformanceDataTypes', params or {})
+        self._execute("mobile: getPerformanceDataTypes", params)
         return self
 
 
@@ -271,7 +274,7 @@ class MobileCommands:
         raise_exception=ShadowstepException,
         exceptions=(NoSuchDriverException, InvalidSessionIdException, StaleElementReferenceException)
     )
-    def get_settings(self, params: dict | list = None) -> 'Shadowstep':
+    def get_settings(self, params: dict[str, Any] | list[Any] | None = None) -> MobileCommands:
         """
         https://github.com/appium/appium-uiautomator2-driver/blob/master/docs/android-mobile-gestures.md#mobile getsettings
         Execute mobile: getSettings command.
@@ -283,7 +286,7 @@ class MobileCommands:
             Shadowstep: Self for method chaining.
         """
         self.logger.debug(f"{get_current_func_name()}")
-        self._execute('mobile: getSettings', params or {})
+        self._execute("mobile: getSettings", params)
         return self
 
 
@@ -293,7 +296,7 @@ class MobileCommands:
         raise_exception=ShadowstepException,
         exceptions=(NoSuchDriverException, InvalidSessionIdException, StaleElementReferenceException)
     )
-    def hide_keyboard(self, params: dict | list = None) -> 'Shadowstep':
+    def hide_keyboard(self, params: dict[str, Any] | list[Any] | None = None) -> MobileCommands:
         """
         https://github.com/appium/appium-uiautomator2-driver/blob/master/docs/android-mobile-gestures.md#mobile hidekeyboard
         Execute mobile: hideKeyboard command.
@@ -305,7 +308,7 @@ class MobileCommands:
             Shadowstep: Self for method chaining.
         """
         self.logger.debug(f"{get_current_func_name()}")
-        self._execute('mobile: hideKeyboard', params or {})
+        self._execute("mobile: hideKeyboard", params)
         return self
 
 
@@ -315,7 +318,7 @@ class MobileCommands:
         raise_exception=ShadowstepException,
         exceptions=(NoSuchDriverException, InvalidSessionIdException, StaleElementReferenceException)
     )
-    def install_app(self, params: dict | list = None) -> 'Shadowstep':
+    def install_app(self, params: dict[str, Any] | list[Any] | None = None) -> MobileCommands:
         """
         https://github.com/appium/appium-uiautomator2-driver/blob/master/docs/android-mobile-gestures.md#mobile installapp
         Execute mobile: installApp command.
@@ -327,7 +330,7 @@ class MobileCommands:
             Shadowstep: Self for method chaining.
         """
         self.logger.debug(f"{get_current_func_name()}")
-        self._execute('mobile: installApp', params or {})
+        self._execute("mobile: installApp", params)
         return self
 
 
@@ -337,7 +340,7 @@ class MobileCommands:
         raise_exception=ShadowstepException,
         exceptions=(NoSuchDriverException, InvalidSessionIdException, StaleElementReferenceException)
     )
-    def is_appinstalled(self, params: dict | list = None) -> 'Shadowstep':
+    def is_appinstalled(self, params: dict[str, Any] | list[Any] | None = None) -> MobileCommands:
         """
         https://github.com/appium/appium-uiautomator2-driver/blob/master/docs/android-mobile-gestures.md#mobile isappinstalled
         Execute mobile: isAppInstalled command.
@@ -349,7 +352,7 @@ class MobileCommands:
             Shadowstep: Self for method chaining.
         """
         self.logger.debug(f"{get_current_func_name()}")
-        self._execute('mobile: isAppInstalled', params or {})
+        self._execute("mobile: isAppInstalled", params)
         return self
 
 
@@ -359,7 +362,7 @@ class MobileCommands:
         raise_exception=ShadowstepException,
         exceptions=(NoSuchDriverException, InvalidSessionIdException, StaleElementReferenceException)
     )
-    def longpress_key(self, params: dict | list = None) -> 'Shadowstep':
+    def longpress_key(self, params: dict[str, Any] | list[Any] | None = None) -> MobileCommands:
         """
         https://github.com/appium/appium-uiautomator2-driver/blob/master/docs/android-mobile-gestures.md#mobile longpresskey
         Execute mobile: longPressKey command.
@@ -371,7 +374,7 @@ class MobileCommands:
             Shadowstep: Self for method chaining.
         """
         self.logger.debug(f"{get_current_func_name()}")
-        self._execute('mobile: longPressKey', params or {})
+        self._execute("mobile: longPressKey", params)
         return self
 
 
@@ -381,7 +384,7 @@ class MobileCommands:
         raise_exception=ShadowstepException,
         exceptions=(NoSuchDriverException, InvalidSessionIdException, StaleElementReferenceException)
     )
-    def open_notifications(self, params: dict | list = None) -> 'Shadowstep':
+    def open_notifications(self, params: dict[str, Any] | list[Any] | None = None) -> MobileCommands:
         """
         https://github.com/appium/appium-uiautomator2-driver/blob/master/docs/android-mobile-gestures.md#mobile opennotifications
         Execute mobile: openNotifications command.
@@ -393,7 +396,7 @@ class MobileCommands:
             Shadowstep: Self for method chaining.
         """
         self.logger.debug(f"{get_current_func_name()}")
-        self._execute('mobile: openNotifications', params or {})
+        self._execute("mobile: openNotifications", params)
         return self
 
 
@@ -403,7 +406,7 @@ class MobileCommands:
         raise_exception=ShadowstepException,
         exceptions=(NoSuchDriverException, InvalidSessionIdException, StaleElementReferenceException)
     )
-    def open_settings(self, params: dict | list = None) -> 'Shadowstep':
+    def open_settings(self, params: dict[str, Any] | list[Any] | None = None) -> MobileCommands:
         """
         https://github.com/appium/appium-uiautomator2-driver/blob/master/docs/android-mobile-gestures.md#mobile opensettings
         Execute mobile: openSettings command.
@@ -415,7 +418,7 @@ class MobileCommands:
             Shadowstep: Self for method chaining.
         """
         self.logger.debug(f"{get_current_func_name()}")
-        self._execute('mobile: openSettings', params or {})
+        self._execute("mobile: openSettings", params)
         return self
 
 
@@ -425,7 +428,7 @@ class MobileCommands:
         raise_exception=ShadowstepException,
         exceptions=(NoSuchDriverException, InvalidSessionIdException, StaleElementReferenceException)
     )
-    def press_key(self, params: dict | list = None) -> 'Shadowstep':
+    def press_key(self, params: dict[str, Any] | list[Any] | None = None) -> MobileCommands:
         """
         https://github.com/appium/appium-uiautomator2-driver/blob/master/docs/android-mobile-gestures.md#mobile presskey
         Execute mobile: pressKey command.
@@ -437,7 +440,7 @@ class MobileCommands:
             Shadowstep: Self for method chaining.
         """
         self.logger.debug(f"{get_current_func_name()}")
-        self._execute('mobile: pressKey', params or {})
+        self._execute("mobile: pressKey", params)
         return self
 
 
@@ -447,7 +450,7 @@ class MobileCommands:
         raise_exception=ShadowstepException,
         exceptions=(NoSuchDriverException, InvalidSessionIdException, StaleElementReferenceException)
     )
-    def query_appstate(self, params: dict | list = None) -> 'Shadowstep':
+    def query_appstate(self, params: dict[str, Any] | list[Any] | None = None) -> MobileCommands:
         """
         https://github.com/appium/appium-uiautomator2-driver/blob/master/docs/android-mobile-gestures.md#mobile queryappstate
         Execute mobile: queryAppState command.
@@ -459,7 +462,7 @@ class MobileCommands:
             Shadowstep: Self for method chaining.
         """
         self.logger.debug(f"{get_current_func_name()}")
-        self._execute('mobile: queryAppState', params or {})
+        self._execute("mobile: queryAppState", params)
         return self
 
 
@@ -469,7 +472,7 @@ class MobileCommands:
         raise_exception=ShadowstepException,
         exceptions=(NoSuchDriverException, InvalidSessionIdException, StaleElementReferenceException)
     )
-    def remove_app(self, params: dict | list = None) -> 'Shadowstep':
+    def remove_app(self, params: dict[str, Any] | list[Any] | None = None) -> MobileCommands:
         """
         https://github.com/appium/appium-uiautomator2-driver/blob/master/docs/android-mobile-gestures.md#mobile removeapp
         Execute mobile: removeApp command.
@@ -481,7 +484,7 @@ class MobileCommands:
             Shadowstep: Self for method chaining.
         """
         self.logger.debug(f"{get_current_func_name()}")
-        self._execute('mobile: removeApp', params or {})
+        self._execute("mobile: removeApp", params)
         return self
 
 
@@ -491,7 +494,7 @@ class MobileCommands:
         raise_exception=ShadowstepException,
         exceptions=(NoSuchDriverException, InvalidSessionIdException, StaleElementReferenceException)
     )
-    def replaceelementvalue(self, params: dict | list = None) -> 'Shadowstep':
+    def replaceelementvalue(self, params: dict[str, Any] | list[Any] | None = None) -> MobileCommands:
         """
         https://github.com/appium/appium-uiautomator2-driver/blob/master/docs/android-mobile-gestures.md#mobile replaceelementvalue
         Execute mobile: replaceElementValue command.
@@ -503,7 +506,7 @@ class MobileCommands:
             Shadowstep: Self for method chaining.
         """
         self.logger.debug(f"{get_current_func_name()}")
-        self._execute('mobile: replaceElementValue', params or {})
+        self._execute("mobile: replaceElementValue", params)
         return self
 
 
@@ -513,7 +516,7 @@ class MobileCommands:
         raise_exception=ShadowstepException,
         exceptions=(NoSuchDriverException, InvalidSessionIdException, StaleElementReferenceException)
     )
-    def scroll_back_to(self, params: dict | list = None) -> 'Shadowstep':
+    def scroll_back_to(self, params: dict[str, Any] | list[Any] | None = None) -> MobileCommands:
         """
         https://github.com/appium/appium-uiautomator2-driver/blob/master/docs/android-mobile-gestures.md#mobile scrollbackto
         Execute mobile: scrollBackTo command.
@@ -525,7 +528,7 @@ class MobileCommands:
             Shadowstep: Self for method chaining.
         """
         self.logger.debug(f"{get_current_func_name()}")
-        self._execute('mobile: scrollBackTo', params or {})
+        self._execute("mobile: scrollBackTo", params)
         return self
 
 
@@ -535,7 +538,7 @@ class MobileCommands:
         raise_exception=ShadowstepException,
         exceptions=(NoSuchDriverException, InvalidSessionIdException, StaleElementReferenceException)
     )
-    def send_sms(self, params: dict | list = None) -> 'Shadowstep':
+    def send_sms(self, params: dict[str, Any] | list[Any] | None = None) -> MobileCommands:
         """
         https://github.com/appium/appium-uiautomator2-driver/blob/master/docs/android-mobile-gestures.md#mobile sendsms
         Execute mobile: sendSMS command.
@@ -547,7 +550,7 @@ class MobileCommands:
             Shadowstep: Self for method chaining.
         """
         self.logger.debug(f"{get_current_func_name()}")
-        self._execute('mobile: sendSMS', params or {})
+        self._execute("mobile: sendSMS", params)
         return self
 
 
@@ -557,7 +560,7 @@ class MobileCommands:
         raise_exception=ShadowstepException,
         exceptions=(NoSuchDriverException, InvalidSessionIdException, StaleElementReferenceException)
     )
-    def set_clipboard(self, params: dict | list = None) -> 'Shadowstep':
+    def set_clipboard(self, params: dict[str, Any] | list[Any] | None = None) -> MobileCommands:
         """
         https://github.com/appium/appium-uiautomator2-driver/blob/master/docs/android-mobile-gestures.md#mobile setclipboard
         Execute mobile: setClipboard command.
@@ -569,7 +572,7 @@ class MobileCommands:
             Shadowstep: Self for method chaining.
         """
         self.logger.debug(f"{get_current_func_name()}")
-        self._execute('mobile: setClipboard', params or {})
+        self._execute("mobile: setClipboard", params)
         return self
 
 
@@ -579,7 +582,7 @@ class MobileCommands:
         raise_exception=ShadowstepException,
         exceptions=(NoSuchDriverException, InvalidSessionIdException, StaleElementReferenceException)
     )
-    def set_text(self, params: dict | list = None) -> 'Shadowstep':
+    def set_text(self, params: dict[str, Any] | list[Any] | None = None) -> MobileCommands:
         """
         https://github.com/appium/appium-uiautomator2-driver/blob/master/docs/android-mobile-gestures.md#mobile settext
         Execute mobile: setText command.
@@ -591,7 +594,7 @@ class MobileCommands:
             Shadowstep: Self for method chaining.
         """
         self.logger.debug(f"{get_current_func_name()}")
-        self._execute('mobile: setText', params or {})
+        self._execute("mobile: setText", params)
         return self
 
 
@@ -601,7 +604,7 @@ class MobileCommands:
         raise_exception=ShadowstepException,
         exceptions=(NoSuchDriverException, InvalidSessionIdException, StaleElementReferenceException)
     )
-    def shell(self, params: dict | list = None) -> 'Shadowstep':
+    def shell(self, params: dict[str, Any] | list[Any] | None = None) -> MobileCommands:
         """
         https://github.com/appium/appium-uiautomator2-driver/blob/master/docs/android-mobile-gestures.md#mobile shell
         Execute mobile: shell command.
@@ -613,7 +616,7 @@ class MobileCommands:
             Shadowstep: Self for method chaining.
         """
         self.logger.debug(f"{get_current_func_name()}")
-        self._execute('mobile: shell', params or {})
+        self._execute("mobile: shell", params)
         return self
 
 
@@ -623,7 +626,7 @@ class MobileCommands:
         raise_exception=ShadowstepException,
         exceptions=(NoSuchDriverException, InvalidSessionIdException, StaleElementReferenceException)
     )
-    def start_activity(self, params: dict | list = None) -> 'Shadowstep':
+    def start_activity(self, params: dict[str, Any] | list[Any] | None = None) -> MobileCommands:
         """
         https://github.com/appium/appium-uiautomator2-driver/blob/master/docs/android-mobile-gestures.md#mobile startactivity
         Execute mobile: startActivity command.
@@ -635,7 +638,7 @@ class MobileCommands:
             Shadowstep: Self for method chaining.
         """
         self.logger.debug(f"{get_current_func_name()}")
-        self._execute('mobile: startActivity', params or {})
+        self._execute("mobile: startActivity", params)
         return self
 
 
@@ -645,7 +648,7 @@ class MobileCommands:
         raise_exception=ShadowstepException,
         exceptions=(NoSuchDriverException, InvalidSessionIdException, StaleElementReferenceException)
     )
-    def start_logsbroadcast(self, params: dict | list = None) -> 'Shadowstep':
+    def start_logsbroadcast(self, params: dict[str, Any] | list[Any] | None = None) -> MobileCommands:
         """
         https://github.com/appium/appium-uiautomator2-driver/blob/master/docs/android-mobile-gestures.md#mobile startlogsbroadcast
         Execute mobile: startLogsBroadcast command.
@@ -657,7 +660,7 @@ class MobileCommands:
             Shadowstep: Self for method chaining.
         """
         self.logger.debug(f"{get_current_func_name()}")
-        self._execute('mobile: startLogsBroadcast', params or {})
+        self._execute("mobile: startLogsBroadcast", params)
         return self
 
 
@@ -667,7 +670,7 @@ class MobileCommands:
         raise_exception=ShadowstepException,
         exceptions=(NoSuchDriverException, InvalidSessionIdException, StaleElementReferenceException)
     )
-    def stop_logsbroadcast(self, params: dict | list = None) -> 'Shadowstep':
+    def stop_logsbroadcast(self, params: dict[str, Any] | list[Any] | None = None) -> MobileCommands:
         """
         https://github.com/appium/appium-uiautomator2-driver/blob/master/docs/android-mobile-gestures.md#mobile stoplogsbroadcast
         Execute mobile: stopLogsBroadcast command.
@@ -679,7 +682,7 @@ class MobileCommands:
             Shadowstep: Self for method chaining.
         """
         self.logger.debug(f"{get_current_func_name()}")
-        self._execute('mobile: stopLogsBroadcast', params or {})
+        self._execute("mobile: stopLogsBroadcast", params)
         return self
 
 
@@ -689,7 +692,7 @@ class MobileCommands:
         raise_exception=ShadowstepException,
         exceptions=(NoSuchDriverException, InvalidSessionIdException, StaleElementReferenceException)
     )
-    def terminate_app(self, params: dict | list = None) -> 'Shadowstep':
+    def terminate_app(self, params: dict[str, Any] | list[Any] | None = None) -> MobileCommands:
         """
         https://github.com/appium/appium-uiautomator2-driver/blob/master/docs/android-mobile-gestures.md#mobile terminateapp
         Execute mobile: terminateApp command.
@@ -701,7 +704,7 @@ class MobileCommands:
             Shadowstep: Self for method chaining.
         """
         self.logger.debug(f"{get_current_func_name()}")
-        self._execute('mobile: terminateApp', params or {})
+        self._execute("mobile: terminateApp", params)
         return self
 
 
@@ -711,7 +714,7 @@ class MobileCommands:
         raise_exception=ShadowstepException,
         exceptions=(NoSuchDriverException, InvalidSessionIdException, StaleElementReferenceException)
     )
-    def togglelocationservices(self, params: dict | list = None) -> 'Shadowstep':
+    def togglelocationservices(self, params: dict[str, Any] | list[Any] | None = None) -> MobileCommands:
         """
         https://github.com/appium/appium-uiautomator2-driver/blob/master/docs/android-mobile-gestures.md#mobile togglelocationservices
         Execute mobile: toggleLocationServices command.
@@ -723,7 +726,7 @@ class MobileCommands:
             Shadowstep: Self for method chaining.
         """
         self.logger.debug(f"{get_current_func_name()}")
-        self._execute('mobile: toggleLocationServices', params or {})
+        self._execute("mobile: toggleLocationServices", params)
         return self
 
 
@@ -733,7 +736,7 @@ class MobileCommands:
         raise_exception=ShadowstepException,
         exceptions=(NoSuchDriverException, InvalidSessionIdException, StaleElementReferenceException)
     )
-    def update_settings(self, params: dict | list = None) -> 'Shadowstep':
+    def update_settings(self, params: dict[str, Any] | list[Any] | None = None) -> MobileCommands:
         """
         https://github.com/appium/appium-uiautomator2-driver/blob/master/docs/android-mobile-gestures.md#mobile updatesettings
         Execute mobile: updateSettings command.
@@ -745,7 +748,7 @@ class MobileCommands:
             Shadowstep: Self for method chaining.
         """
         self.logger.debug(f"{get_current_func_name()}")
-        self._execute('mobile: updateSettings', params or {})
+        self._execute("mobile: updateSettings", params)
         return self
 
 
@@ -755,7 +758,7 @@ class MobileCommands:
         raise_exception=ShadowstepException,
         exceptions=(NoSuchDriverException, InvalidSessionIdException, StaleElementReferenceException)
     )
-    def get_text(self, params: dict | list = None) -> 'Shadowstep':
+    def get_text(self, params: dict[str, Any] | list[Any] | None = None) -> MobileCommands:
         """
         https://github.com/appium/appium-uiautomator2-driver/blob/master/docs/android-mobile-gestures.md#mobile gettext
         Execute mobile: getText command.
@@ -767,7 +770,7 @@ class MobileCommands:
             Shadowstep: Self for method chaining.
         """
         self.logger.debug(f"{get_current_func_name()}")
-        self._execute('mobile: getText', params or {})
+        self._execute("mobile: getText", params)
         return self
 
 
@@ -777,7 +780,7 @@ class MobileCommands:
         raise_exception=ShadowstepException,
         exceptions=(NoSuchDriverException, InvalidSessionIdException, StaleElementReferenceException)
     )
-    def performeditoraction(self, params: dict | list = None) -> 'Shadowstep':
+    def performeditoraction(self, params: dict[str, Any] | list[Any] | None = None) -> MobileCommands:
         """
         https://github.com/appium/appium-uiautomator2-driver/blob/master/docs/android-mobile-gestures.md#mobile performeditoraction
         Execute mobile: performEditorAction command.
@@ -789,10 +792,12 @@ class MobileCommands:
             Shadowstep: Self for method chaining.
         """
         self.logger.debug(f"{get_current_func_name()}")
-        self._execute('mobile: performEditorAction', params or {})
+        self._execute("mobile: performEditorAction", params)
         return self
 
-    def _execute(self, name: str, params: dict | list) -> None:
+    def _execute(self, name: str, params: dict[str, Any] | list[Any] | None) -> None:
         # https://github.com/appium/appium-uiautomator2-driver/blob/master/docs/android-mobile-gestures.md
         self.driver = WebDriverSingleton.get_driver()
-        self.driver.execute_script(name, params)
+        if self.driver is None:
+            raise ShadowstepException("WebDriver is not available")
+        self.driver.execute_script(name, params or {})
